# -*- coding: utf-8 -*-

"""
    The extraction module
    ======================
"""

import logging
import os

import cv2
import imageio.v2 as iio
from arkindex import ArkindexClient, options_from_env
from tqdm import tqdm

from dan.datasets.extract.arkindex_utils import retrieve_corpus, retrieve_subsets

logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


IMAGES_DIR = "./images/"  # Path to the images directory.
LABELS_DIR = "./labels/"  # Path to the labels directory.

# Layout string to token
SEM_MATCHING_TOKENS_STR = {
    "INTITULE": "ⓘ",
    "DATE": "ⓓ",
    "COTE_SERIE": "ⓢ",
    "ANALYSE_COMPL.": "ⓒ",
    "PRECISIONS_SUR_COTE": "ⓟ",
    "COTE_ARTICLE": "ⓐ",
}

# Layout begin-token to end-token
SEM_MATCHING_TOKENS = {"ⓘ": "Ⓘ", "ⓓ": "Ⓓ", "ⓢ": "Ⓢ", "ⓒ": "Ⓒ", "ⓟ": "Ⓟ", "ⓐ": "Ⓐ"}


def add_extract_parser(subcommands) -> None:
    parser = subcommands.add_parser(
        "extract",
        description=__doc__,
        help=__doc__,
    )
    # Required arguments.
    parser.add_argument(
        "--corpus",
        type=str,
        help="Name of the corpus from which the data will be retrieved.",
        required=True,
    )
    parser.add_argument(
        "--element-type",
        nargs="+",
        type=str,
        help="Type of elements to retrieve",
        required=True,
    )
    parser.add_argument(
        "--parents-types",
        nargs="+",
        type=str,
        help="Type of parents of the elements.",
        required=True,
    )
    parser.add_argument(
        "--output-dir",
        type=str,
        help="Path to the output directory.",
        required=True,
    )

    # Optional arguments.
    parser.add_argument(
        "--parents-names",
        nargs="+",
        type=str,
        help="Names of parents of the elements.",
        default=None,
    )
    parser.add_argument(
        "--no-entities", action="store_true", help="Extract text without entities"
    )

    parser.add_argument(
        "--use-existing-split",
        action="store_true",
        help="Do not partition pages into train/val/test",
    )

    parser.add_argument(
        "--train-prob", type=float, default=0.7, help="Training set probability"
    )

    parser.add_argument(
        "--val-prob", type=float, default=0.15, help="Validation set probability"
    )
    parser.set_defaults(func=run)


def run(
    corpus,
    element_type,
    parents_types,
    output_dir,
    parents_names,
    no_entities,
    use_existing_split,
    train_prob,
    val_prob,
):
    # Get and initialize the parameters.
    os.makedirs(IMAGES_DIR, exist_ok=True)
    os.makedirs(LABELS_DIR, exist_ok=True)

    # Login to arkindex.
    client = ArkindexClient(**options_from_env())

    corpus = retrieve_corpus(client, corpus)
    subsets = retrieve_subsets(client, corpus, parents_types, parents_names)

    # Iterate over the subsets to find the page images and labels.
    for subset in subsets:

        os.makedirs(os.path.join(output_dir, IMAGES_DIR, subset["name"]), exist_ok=True)
        os.makedirs(os.path.join(output_dir, LABELS_DIR, subset["name"]), exist_ok=True)

        for page in tqdm(
            client.paginate(
                "ListElementChildren", id=subset["id"], type="page", recursive=True
            ),
            desc="Set " + subset["name"],
        ):

            image = iio.imread(page["zone"]["url"])
            cv2.imwrite(
                os.path.join(
                    output_dir, IMAGES_DIR, subset["name"], f"{page['id']}.jpg"
                ),
                cv2.cvtColor(image, cv2.COLOR_BGR2RGB),
            )

            tr = client.request(
                "ListTranscriptions", id=page["id"], worker_version=None
            )["results"]
            tr = [one for one in tr if one["worker_version_id"] is None]
            assert len(tr) == 1, page["id"]

            for one_tr in tr:
                ent = client.request("ListTranscriptionEntities", id=one_tr["id"])[
                    "results"
                ]
                ent = [one for one in ent if one["worker_version_id"] is None]
                if len(ent) == 0:
                    continue
                else:
                    text = one_tr["text"]

            new_text = text
            count = 0
            for e in ent:
                start_token = SEM_MATCHING_TOKENS_STR[e["entity"]["metas"]["subtype"]]
                end_token = SEM_MATCHING_TOKENS[start_token]
                new_text = (
                    new_text[: count + e["offset"]]
                    + start_token
                    + new_text[count + e["offset"] :]
                )
                count += 1
                new_text = (
                    new_text[: count + e["offset"] + e["length"]]
                    + end_token
                    + new_text[count + e["offset"] + e["length"] :]
                )
                count += 1

            with open(
                os.path.join(
                    output_dir, LABELS_DIR, subset["name"], f"{page['id']}.txt"
                ),
                "w",
            ) as f:
                f.write(new_text)
